@@ -2,6 +2,9 @@
 username = 'guest'
 email = 'guest@example.org'
 password = 'passw0rd'
+username2 = 'guest2'
+email2 = 'guest2@example.org'
+password2 = 'passw2rd'
 category = 'dummy-category'
 category2 = 'dummy2-category'
 tag = 'dummy-tag'
